class Tomify::Setting < Tomify.models.base
  validates_presence_of :type, :name
  validates_uniqueness_of :name
  validates_format_of :name, with: /\A[a-z_]+\z/, message: "can only contain lowercase letters and underscores"
  validate :type_valid?, on: :update
  validate :name_valid?, on: :update
  validate :public_valid?, on: :update

  after_commit :update_config
  before_destroy { |record| throw :abort if record.name.in? self.class.required_settings }

  def self.required_settings
    ["allow_signup", "aws", "name", "email", "timezone"]
  end

  def self.for_env
    Tomify.models.setting.where(public: true).as_json(only: [:name, :value])
  end

  private
  def type_valid?
    errors.add(:type, "cannot change") if type_changed?
  end

  def name_valid?
    return true unless name_was.in? self.class.required_settings
    errors.add(:name, "cannot change if required") if name_changed?
  end

  def public_valid?
    return true unless name_was.in? self.class.required_settings
    errors.add(:public, "cannot change if required") if public_changed?
  end

  def update_config
    case name
    when "aws"
      Tomify::CarrierwaveHelper.load_config
    when "email"
      Tomify::EmailHelper.load_config
    when "timezone"
      Tomify::TimezoneHelper.load_config
    end
  end
end
